//+------------------------------------------------------------------+
//|                                                    TimeRange.mq4 |
//|                                               mt4programming.com |
//|                                    http://www.mt4programming.com |
//+------------------------------------------------------------------+
#property copyright "mt4programming.com"
#property link      "http://www.mt4programming.com"
//Owner Michael Tremante 
//version 1.0 by Alex Pyrkov Feb 15, 2013
//version 1.1 by Alex Pyrkov March 06, 2013
//version 1.2 by Alex Pyrkov March 21, 2013
//version 1.3 by Alex Pyrkov Jan 10, 2014
//version 1.4 by Alex Pyrkov Jan 17, 2014
//version 1.5 by Alex Pyrkov Feb 03, 2015

#include <stderror.mqh>
#include <stdlib.mqh>
#define name "TimeRange v 1.5"

extern string comment=name;
extern int MagicNumber=1718191001;
extern int slippage=3;
extern double fixed_lots=0.01;
extern bool useMoneyManagement=true;
extern double RiskPerTrade_percent=3;

extern bool spanOverMidnight=true;
extern int entry_range_from_hours=22;
extern int entry_range_from_minutes=0;
extern int entry_range_to_hours=9;
extern int entry_range_to_minutes=0;
extern int stop_orders_offset_points=10;
extern bool showTimeOnChart=true;
//days of week and times
extern bool Monday=true;
extern bool Tuesday=true;
extern bool Wednesday=true;
extern bool Thursday=true;
extern bool Friday=true;
extern bool Saturday=true;
extern bool Sunday=true;
//breakeven
extern bool use_breakeven=true;
extern double breakeven_threshold_percent=50; 
extern int breakeven_addition_points=20; 
//trailing stop
extern bool useTrailingStop=true;
extern int trailingStop_threshold=200;
extern int trailingStop_trail=100;
//OCO
extern bool useOCO=true;
//Lines
extern color LineColor=Orange;
extern ENUM_LINE_STYLE LineStyle=STYLE_SOLID;
extern int LineWidth=2;

//order repetition and other constants
int repeat=30;//new
int sleep_interval=1000;//new
int underlyingTF=1;
int minunderlyingBars=1002;
//variables
int last_day_entered=-1;
bool tradingDays[7];
bool oco_calculated=false;
string prefix="TR_";
int lineNumber=1;

int init()
{
   last_day_entered=-1;
   tradingDays[0]=Sunday;
   tradingDays[1]=Monday;
   tradingDays[2]=Tuesday;
   tradingDays[3]=Wednesday;
   tradingDays[4]=Thursday;
   tradingDays[5]=Friday;
   tradingDays[6]=Saturday;
   
   oco_calculated=false;
   lineNumber=1;
   return(0);
}

int deinit()
{
   if(showTimeOnChart) Comment("");
   DeleteObjectsByPrefix(prefix);   
   return(0);
}

int start()
{
   double rl=-1;
   double rh=-1;
   int sl=0;
   if(last_day_entered<0) // just initialised
   {
      //initial markup
      datetime last_day_midnight=0;
      for(int i=Bars-1;i>=0;i--)
      {
         if(Midnight(Time[i])>last_day_midnight && tradingDays[TimeDayOfWeek(Time[i])])
         {
            last_day_midnight=Midnight(Time[i]);
            datetime rt=last_day_midnight+entry_range_to_hours*3600+entry_range_to_minutes*60-PeriodSeconds();//latest bar should not be counted
            datetime rf=last_day_midnight+entry_range_from_hours*3600+entry_range_from_minutes*60;
            if(spanOverMidnight)//yesterday, same time
            {
               rf-=24*3600;
            }
            if(rt<GetTime())
            {
               rl=-1;
               rh=-1;
               if(GetHighLow(rf,rt,rl,rh))
               {
                  if(rl>0 && rh>0)
                  {
                     DrawLines(rf,rt,rl,rh);               
                  }
               }
            }
         }
      }
      //end init markup
      if(TimeAfter())//too late for today
      {
         last_day_entered=TimeDay(GetTime());
      }
      else
      {
         last_day_entered=0;//will be able to enter
      } 
   }
   int ubn=iBars(Symbol(),underlyingTF);
   rl=-1;
   rh=-1;
   if(last_day_entered!=TimeDay(GetTime()))// will not enter twice
   {
      if(TimeAfterWithin300())
      {
         DeletePending();
         //do enter
         if(tradingDays[TimeDayOfWeek(GetTime())] && ubn>=minunderlyingBars)
         {      
            rh=GetHighSince(RangeBegins());
            rl=GetLowSince(RangeBegins());
            if(rl>0 && rh>0)
            {
               DrawLines(RangeBegins(),RangeEnds()-PeriodSeconds(),rl,rh);
               sl=(rh-rl+Ask-Bid)/Point;
               rh+=stop_orders_offset_points*Point;
               rl-=stop_orders_offset_points*Point;

               EnterLongStopMarket(rh,sl);
               EnterShortStopMarket(rl,sl);         
               last_day_entered=TimeDay(GetTime());
            }
         }
      }
   }
   Breakeven();
   Trail();
   OCO();
   UpdateComment();
   return(0);
}

bool TimeAfter()
{
   datetime now=GetTime();
   return (TimeCompare(TimeHour(now),TimeMinute(now),entry_range_to_hours,entry_range_to_minutes)>=0);
}

bool TimeAfterWithin300() 
{
   int span=GetTime()-RangeEnds();
   return (span<300 && span>=0);
}

int TimeCompare(int h1,int m1,int h2,int m2)
{
   if(h1>h2) return (1);
   if(h1<h2) return (-1);
   if(m1>m2) return (1);
   if(m1<m2) return (-1);
   return (0);
}

datetime GetTime() // in case we need to change TZ later
{
   return (TimeCurrent());
}

void DeletePending()
{
   //close pending, if any 
   int total=OrdersTotal();
   int toDelete[50];
   ArrayInitialize(toDelete,-1);
   int j=0;
   for(int i=0;i<total;i++)
   {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) continue;
      if(OrderSymbol()!=Symbol()) continue;
      if(OrderCloseTime()!=0) continue;
      if(OrderMagicNumber()!=MagicNumber) continue;
      if(OrderType()==OP_BUYSTOP || OrderType()==OP_BUYLIMIT || OrderType()==OP_SELLLIMIT || OrderType()==OP_SELLSTOP) 
      {
         toDelete[j]=OrderTicket();
         j++;
         if(j>=(ArraySize(toDelete)-1)) ArrayResize(toDelete,ArraySize(toDelete)+50);
      }
   }
   int sz=ArraySize(toDelete);
   for(int k=0;k<sz;k++)
   {
      if(toDelete[k]>0)
      {
         int count=0;
         while(!OrderDelete(toDelete[k]))
         {
            count++;
            if(count<=repeat)
            {
               Sleep(sleep_interval);
            }
            else
            {
               PrintError(Symbol());
               break;
            }
         }
      }
   }
}

void PrintError(string pair)
{
   int err=GetLastError();
   Print("Error# ",err," ",ErrorDescription(err)," for ",pair);
}

void EnterLongStopMarket(double target,int sltp)
{
  int res=-1;
  double sl=target-sltp*Point;
  double tp=target+sltp*Point;
  for(int i=0;(i<repeat) && (res<0);i++)
  {
      RefreshRates();
      res=OrderSend(Symbol(),OP_BUYSTOP,Lots(sltp),target,slippage,sl,tp,comment,MagicNumber,0,Green);
      if(res<0)
      {
        PrintError(Symbol());
        Sleep(sleep_interval);
      }
      else break;
   }
}

void EnterShortStopMarket(double target,int sltp)
{
  int res=-1;
  double sl=target+sltp*Point;
  double tp=target-sltp*Point;
  for(int i=0;(i<repeat) && (res<0);i++)
  {
      RefreshRates();
      res=OrderSend(Symbol(),OP_SELLSTOP,Lots(sltp),target,slippage,sl,tp,comment,MagicNumber,0,Red);
      if(res<0)
      {
        PrintError(Symbol());
        Sleep(sleep_interval);
      }
      else break;
   }
}

datetime TodayMidnight()
{
   datetime now=GetTime();
   return (now-TimeHour(now)*3600-TimeMinute(now)*60-TimeSeconds(now));
}

datetime Midnight(datetime dt)
{
   return (dt-TimeHour(dt)*3600-TimeMinute(dt)*60-TimeSeconds(dt));
}

datetime RangeBegins()
{
   datetime rb=(TodayMidnight()+entry_range_from_hours*3600+entry_range_from_minutes*60);
   if(spanOverMidnight)//yesterday, same time
   {
      rb-=24*3600;
   }
   return (rb);
}

datetime RangeEnds()
{
   return (TodayMidnight()+entry_range_to_hours*3600+entry_range_to_minutes*60);
}

double GetHighSince(datetime dt)
{
   int bn=iBarShift(NULL,underlyingTF,dt,false);
   if(bn>=0)
   {
      return (iHigh(NULL,underlyingTF,iHighest(NULL,underlyingTF,MODE_HIGH,bn,0)));
   }
   return (-1);
}

double GetLowSince(datetime dt)
{
   int bn=iBarShift(NULL,underlyingTF,dt,false);
   if(bn>=0)
   {
      return (iLow(NULL,underlyingTF,iLowest(NULL,underlyingTF,MODE_LOW,bn,0)));
   }
   return (-1);
}

bool GetHighLow(datetime from,datetime to,double& rl,double& rh)
{
   int bnf=iBarShift(NULL,PERIOD_CURRENT,from,false);
   int bnt=iBarShift(NULL,PERIOD_CURRENT,to,false);
   if(bnf>=0 && bnt>=0)
   {
      rh=iHigh(NULL,PERIOD_CURRENT,iHighest(NULL,PERIOD_CURRENT,MODE_HIGH,bnf-bnt,bnt));
      rl=iLow(NULL,PERIOD_CURRENT,iLowest(NULL,PERIOD_CURRENT,MODE_LOW,bnf-bnt,bnt));
      return true;
   }
   return false;

}

double RoundLots(double lots)
{
   double l=lots;
   double maxl=MarketInfo(Symbol(),MODE_MAXLOT);
   double minl=MarketInfo(Symbol(),MODE_MINLOT);
   double stepl=MarketInfo(Symbol(),MODE_LOTSTEP);
   if(l<minl) l=minl;
   if(l>maxl) l=maxl;
   l=MathFloor(l/stepl)*stepl;
   return (l);
}

double LotsForSL(double sl_points,double RiskPerTrade)
{
   double sl_ticks=sl_points*Point/MarketInfo(Symbol(),MODE_TICKSIZE);
   double free=MathMin(AccountBalance(),AccountFreeMargin());
   double lots=RiskPerTrade/100.*free/sl_ticks/MarketInfo(Symbol(),MODE_TICKVALUE);
   //Margin can be insufficient
   double lot_lim=AccountFreeMargin()/MarketInfo(Symbol(),MODE_MARGINREQUIRED);
   return (MathMin(lots,lot_lim));
}

double Lots(int sl)
{
   if(!useMoneyManagement)
   {
      return (fixed_lots);
   }
   double l=LotsForSL(sl,RiskPerTrade_percent);
   l=RoundLots(l);
   return (l);
}

void UpdateComment()
{
   if(showTimeOnChart)
   {
      string exp=" NOT expired";
      if(TimeAfter()) exp=" EXPIRED FOR TODAY";
      else 
      {
         int sec=RangeEnds()-GetTime();
         exp=" Will expire in "+TimeToStr(sec,TIME_SECONDS);
      }
      int dow=TimeDayOfWeek(GetTime());
      string hdow=HumanDayOfWeek(dow);
      if(!tradingDays[dow]) hdow=hdow+" DAY OFF";
      
      int ubn=iBars(Symbol(),underlyingTF);
      string ubs="LOANDING underlying TF...";
      if(ubn>=minunderlyingBars) ubs="Underlying TF is loaded";

      Comment(
      "------------------------\n"
      +" Current time "+TimeToStr(GetTime(),TIME_DATE|TIME_SECONDS)+" "+hdow+ "\n"
      +" Range from "+TimeToStr(RangeBegins(),TIME_DATE|TIME_MINUTES)+" to "+TimeToStr(RangeEnds(),TIME_DATE|TIME_MINUTES)+"\n"
      +exp+"\n"+
      ubs
      );
   }
}
 
string HumanDayOfWeek(int dow)
{
   switch(dow)
   {
      case 0:
         return ("Sunday");
      case 1:
         return ("Monday");
      case 2:
         return ("Tuesday");
      case 3:
         return ("Wednesday");
      case 4:
         return ("Thursday");
      case 5:
         return ("Friday");
      case 6:
         return ("Saturday");

   }
   return ("Unknown");
}

void Breakeven()
{
   if(!use_breakeven) return;
   double sl=-1;
   int total=OrdersTotal();
   for(int n=0;n<total;n++)
   {
      if(OrderSelect(n,SELECT_BY_POS,MODE_TRADES)==false) continue;
      if(OrderSymbol()!=Symbol()) continue;
      if(OrderCloseTime()!=0) continue;
      if(IsEqual(OrderTakeProfit(),0)) continue;
      RefreshRates();
      if(OrderType()==OP_SELL) 
      {
         if((OrderOpenPrice()-Ask)>=((OrderOpenPrice()-OrderTakeProfit())*breakeven_threshold_percent/100))
         {
            sl=OrderOpenPrice()-breakeven_addition_points*Point;
            sl=NormalizeDouble(sl,Digits);
         }
         if((sl<OrderStopLoss() || IsEqual(OrderStopLoss(),0)) && (sl>=0))
         {
            if(OrderModify(OrderTicket(),OrderOpenPrice(),sl,OrderTakeProfit(),0))
               continue;
         }
      }
      if(OrderType()==OP_BUY) 
      {
         
         if((Bid-OrderOpenPrice())>=((OrderTakeProfit()-OrderOpenPrice())*breakeven_threshold_percent/100))
         {
            sl=OrderOpenPrice()+breakeven_addition_points*Point;
            sl=NormalizeDouble(sl,Digits);
         }
         if(sl>OrderStopLoss())
         {
            if(OrderModify(OrderTicket(),OrderOpenPrice(),sl,OrderTakeProfit(),0))
               continue;
         }
      }
   }
}

void Trail()
{
   if(!useTrailingStop) return;
   if(trailingStop_threshold<=0) return;
   if(trailingStop_trail<=0) return;

   double sl=0;
   bool res;
   int total=OrdersTotal();
   for(int n=0;n<total;n++)
   {
      if(OrderSelect(n,SELECT_BY_POS,MODE_TRADES)==false) continue;
      if(OrderSymbol()!=Symbol()) continue;
      if(OrderCloseTime()!=0) continue;
      if(OrderType()==OP_SELL) 
      {
         if(Ask<=(OrderOpenPrice()-trailingStop_threshold*Point) || (!IsEqual(OrderStopLoss(),0) && (OrderStopLoss()<(OrderOpenPrice()-(breakeven_addition_points+1)*Point)) ) )
         {
            sl=Ask+trailingStop_trail*Point;
            if(sl<OrderStopLoss() || IsEqual(OrderStopLoss(),0))
               res=OrderModify(OrderTicket(),OrderOpenPrice(),sl,OrderTakeProfit(),0);
         }
         continue;
      }
      if(OrderType()==OP_BUY) 
      {
         if(Bid>=(OrderOpenPrice()+trailingStop_threshold*Point) || (!IsEqual(OrderStopLoss(),0) && (OrderStopLoss()>(OrderOpenPrice()+(breakeven_addition_points+1)*Point)) ) )
         {
            sl=Bid-trailingStop_trail*Point;
            if(sl>OrderStopLoss())
               res=OrderModify(OrderTicket(),OrderOpenPrice(),sl,OrderTakeProfit(),0);
         }
         continue;
      }
   }
}

bool IsEqual(double val1, double val2,int acc=1)
{
   return (MathAbs(val1-val2)<=(acc*Point));
}

void OCO()
{
   if(!useOCO) return;
   if(TotalNumberOfTrades()==1 && (NumberOfTrades(OP_BUYSTOP)==1 || NumberOfTrades(OP_SELLSTOP)==1))
   {
      if(!oco_calculated)
      {
         int ticket=GetLastClosedOrder();
         if(ticket>=0)
         {
            if(OrderSelect(ticket,SELECT_BY_TICKET))
            {
               if(OrderProfit()>0)
               {
                  Print("OCO cancellation");
                  DeletePending();
               }
               oco_calculated=true;
            }
         }
      }
   }
   else
   {
      oco_calculated=false;
   } 
}

int NumberOfTrades(int type)
{
   int counter=0;
   int total=OrdersTotal();
   for(int n=0;n<total;n++)
   {
      if(OrderSelect(n,SELECT_BY_POS,MODE_TRADES)==false) continue;
      if(OrderSymbol()!=Symbol()) continue;
      if(OrderCloseTime()!=0) continue;
      if(OrderType()==type) 
      {
         counter++;
      }
   }
   return (counter);
}

int TotalNumberOfTrades()
{
   int counter=0;
   int total=OrdersTotal();
   for(int n=0;n<total;n++)
   {
      if(OrderSelect(n,SELECT_BY_POS,MODE_TRADES)==false) continue;
      if(OrderSymbol()!=Symbol()) continue;
      if(OrderCloseTime()!=0) continue;
      counter++;
   }
   return (counter);
}

int GetLastClosedOrder()
{
   datetime lasttime=0;
   int ticket=-1;
   int total=OrdersHistoryTotal();
   for(int n=0;n<total;n++)
   {
      if(OrderSelect(n,SELECT_BY_POS,MODE_HISTORY)==false) continue;
      if(OrderSymbol()!=Symbol()) continue;
      if(OrderType()==OP_BUY || OrderType()==OP_SELL)
      {
         if(OrderCloseTime()>lasttime)
         {
            lasttime=OrderCloseTime();
            ticket=OrderTicket();
         }
      }
   }
   return (ticket);
}

void DeleteObjectsByPrefix(string pref)
{
   bool something_removed=true;
   while(something_removed)
   {
      something_removed=false;      
      int tot=ObjectsTotal();   
      for(int i=0;i<tot;i++)
      {
         string nm=ObjectName(i);
         if(StringFind(nm,pref)>=0) 
         {
            ObjectDelete(nm);
            something_removed=true;
            break;      
         }
      }
   }
}

void DrawLines(datetime from,datetime to,double l,double h)
{
   DrawLine(from,to,l);
   DrawLine(from,to,h);
}

void DrawLine(datetime from,datetime to,double p)
{
   string nm=prefix+IntegerToString(lineNumber);
   lineNumber++;
   if(ObjectCreate(nm,OBJ_TREND,0,from,p,to,p))
   {
      ObjectSetInteger(0,nm,OBJPROP_COLOR,LineColor);
      ObjectSetInteger(0,nm,OBJPROP_WIDTH,LineWidth);            
      ObjectSetInteger(0,nm,OBJPROP_STYLE,LineStyle);
      ObjectSetInteger(0,nm,OBJPROP_RAY,false);
      ObjectSetInteger(0,nm,OBJPROP_RAY_LEFT,false);
      ObjectSetInteger(0,nm,OBJPROP_RAY_RIGHT,false);
      ObjectSetInteger(0,nm,OBJPROP_SELECTABLE,false);
   }
}